#!/opt/perl/bin/perl
#
# This script is generated by the trapapp web application.
# It is used as an automatic action in trapd.conf to forward events 
# to OVO as opc messages.
#
# Note:
# * This script requires that trapd.conf be configured to pass the following
#	parameters:
#
#	1. trapnum
#	2. agent name
# 	3 onward. all varbinds
#
# * The severity of traps may be assigned dynamically based on a varbind
# 	in the trap.
#
# Authior: Andy Yuen
# Date: August 2005.
#
##################################################

my $OPCMSG_FMT =qq(/opt/OV/bin/OpC/opcmsg severity=%s application="%s" ) .
	qq(object="%s" msg_text="--AaYy--%s" msg_grp="%s" node=%s);


# Translate from specific trap number to trapd description and other info
# used in creating/sending of opcmsg
my %TRAPDESCR = (
#	key		1. text format, 
#			2. correlator format,
#			3. index to varbind containing severity if a number, 
#				otherwise, severity override, 
#			4. if it is a number in 3, contains hash for status conversion
"15"	=>	[
"varbind1=~~1~~, varbind2=~~2~~ ",
"~~1~~-~~2~~ ",
"1",
{"normal"=>"normal","warning"=>"warning","minor"=>"minor","major"=>"major","critical"=>"critical"}],
);

# severity translation from severity pointer to OVO
# note that any non-negative value mean dynamically assigned severity which
# is handled separately

# check number of arguments
if (scalar(@ARGV) < 2) {
	printUsage();
	exit 1;
}

# get the parameters from the command line
my ($trapnum, @parms) = @ARGV;


# check if we want to handle this trap
if (!exists($TRAPDESCR{$trapnum})) {
	exit 2;
}

# assemble text message
my $data = $TRAPDESCR{$trapnum};
my ($summary, $corr, $sevIdx, $trans) = @$data;

# create hash for parameter substitution
my %info = ();
for (my $i = 0; $i < scalar(@parms) ; $i++) {
	$info{$i} = $parms[$i];
}

# create opc message text based on user-defined format
my $text = format_text($summary, \%info);

# create correlator based on user-defined format
my $obj = format_text($corr, \%info);

# translate MIB-specific severity to OpenView severity where required
my $sev;
if ($sevIdx =~ /^\d+$/) {
	# if a number, index into varbind (dynamic severity translation)
	$sev = $trans->{$info{$sevIdx}};
	if (!defined($sev)) {
		printf "cannot translate severity: index=%s, val=%s\n", $sevIdx, $info{$sevIdx};
	}
}
else {
	# fixed severity
	$sev = $sevIdx;
}

# create opcmsg comand
my $cmd = sprintf $OPCMSG_FMT, 
	$sev,			# severity
	"eHealth",		# application
	$obj,			# object
	$text,			# message
	"eHelath",	# msg_group
	format_text("~~0~~", \%info);		# node name

printf "cmd: %s\n", $cmd;

# execute the opcmsg command to relate the message to the OVO server
#system($cmd);

exit 0;



##########################################
# start of subroutines
#########################################
# print usgae message
sub printUsage {
	print "Usage: ./gen/concord-diagmon.mib.pl trapnum agent param1 param2 ... paramN\n"
}


# *******************************************************************
# format text by substituting values into special markers
sub format_text {
	my ($text, $fillings) = @_;
	# replace quoted words with value in %$fillings hash
	$text =~ s{~~(.*?)~~}
			{exists($fillings->{$1})
				? $fillings->{$1}
				: ""
		}gsex;
	return $text;
}

