#!/opt/perl/bin/perl
#
# trappapp web application for trap customization on NNM
# Version 1.0
#
# Author: Andy Yuen
# Email: andyyuen@ozemail.com.au
# Date: August 2005

# unbuffer output so that we can see it as it comes 
# out, rather than waiting for buffers to flush
$| = 1;

use lib "./";

use CGI;
use POSIX;
use Config::IniFiles;
use MIBFSM;

# ******************************************************************
my $cookiename = "OVTRAPAPP";
my $datadir = "./data/";
my $appini = "trapapp.ini";
my $opcmsg = "opcmsg";
my $myappname = "trapapp";

my $delimiter = "~";

my $trapsection = "trap";
my $formatsection = "format";
my $alarmsection = "alarm";
my $clearsection = "clearing";
my $opcinfosection = "opcinfo";

my @sections = (
	$trapsection,
	$formatsection,
	$alarmsection,
	# leave $clearsection out because the config is valid without it
	#$clearsection,	
	$opcinfosection
	);

my @opcinfo = (
	"appname",
	"messagegroup",
	"nodename"
	);

my $mibname = "mibname";
my $notimplemented = "Function not implemented\n";

my $ignore_state = "ignore";
my $alarm_state = "alarm";
my $clear_state = "clearing";

my $sev_normal = "normal";
my $sev_warning = "warning";
my $sev_minor = "minor";
my $sev_major = "major";
my $sev_critical = "critical";

my @severities = (
	$sev_normal,
	$sev_warning,
	$sev_minor,
	$sev_major,
	$sev_critical
	);

my %jump_table = (
	'UPLOADFILE'	=>	\&uploadfile,
	'UPLOAD'		=>	\&uploadfile,	# display form
	'SELECTFILE'	=>	\&selectfile,
	'CONFIGFILE'	=>	\&configfile,
	'GENERATE'		=>	\&generate,
	'GENSCRIPT'		=>	\&generate,
	'EDITFILE'		=>	\&editfile,
	'DISPLAY'		=>	\&editfile		# display form
);

# generated code format
my $trapdesc_format = qq("%d"\t=>\t[\n"%s",\n"%s",\n"%s",\n%s],);
my $hash_format = qq("%s"=>"%s");

# note that this script does not make use of cookies and sessions keys although
# it generates them
my $query = new CGI;
my $user_key = $query->cookie($cookiename);

if ($query->param("ID") eq "UPLOADFILE") {
	# set the cookie
	($session_key = tmpnam()) =~ s/\//a/g;
	print $query->header(-cookie=>$query->cookie(-name=>$cookiename,
			-value=>"$session_key"));
}
else {
	print $query->header;
}


# start html document
start_doco($query);

#debug only
#print "<PRE>cookie=$session_key and $user_key</PRE>";


unless ($query->param) {
   print_prompt($query);
} else {
   do_work($query);
}

# end html document
end_doco($query);

# ******************************************************************
# display default html page
sub print_prompt {

    my($q) = @_;
	# display upload screen
	selectfile($q);

}

# ******************************************************************
# invoke the appropriate form handler
sub do_work {

	my($q) = @_;
	my($self) = $q->url;
	$q->import_names("Q");

	if ($jump_table{$Q::ID}) {
		$code = $jump_table{$Q::ID};
		&$code($q);
	}
	else {
		print 
			"<H1>This page ($Q::ID) has been retired.</H>";
	}

}

# ******************************************************************
# file upload processing
sub uploadfile {

	my($q) = @_;
	$q->import_names("Q");

	if ($Q::ID eq "UPLOADFILE") {
		#handle the upload
		# convert the upload filename into a file handle as the 1st parameter
		my $filename = $datadir . $q->param("uploadfile") . ".conf";

		# check if config file alread exists
		if (-e $filename) {
			close($q->param("uploadfile"));
			$q->param($mibname, $q->param("uploadfile"));
			editfile($q);
			return;
		}

		copy_text_file($q->upload("uploadfile"), $filename);

		# process uploaded file
		my $fsm = new MIBFSM;
		#print $filename;
		$fsm->getMibInfo($filename);

		# display and save the info
		save_traps($q, $fsm, $q->param("uploadfile"));
	}
	else {
		print 
			template("header.tpl", {DESC => "Upload File"}),
			template("menu.tpl"); 

		print
			$q->start_multipart_form,
			template("upload.tpl"),
			$q->end_form();
	}
}

# ******************************************************************
# edit config processing
sub editfile {

	my($q, $ini) = @_;
	$q->import_names("Q");

	print 
		template("header.tpl", {DESC => "Edit File"}),
		template("menu2.tpl", {MIBNAME=>$Q::mibname});

	# get config info
	if (!$ini) {
		$ini = new Config::IniFiles(-file=>$datadir . $Q::mibname . ".conf");
	}

	# create format section if not already present
	if (!$ini->SectionExists($formatsection)) {
		create_format_secton($ini);
	}

	if ($Q::ID eq "EDITFILE") {
		#process edited info
		#print $notimplemented;

		# display the trap info
		# get trap number
		my @tmp = split(" ", $Q::config);
		my $trapnum = $tmp[2];
		my @traps = split($delimiter, $ini->val($trapsection, $trapnum));
		my $trapname = shift @traps;
		my @formats = split($delimiter, $ini->val($formatsection, $trapnum));
		my $event = shift @formats;

		print
			$q->start_multipart_form, 
			template("config_header.tpl",
				{DESC=>$trapnum . ": " . $trapname, $event=>"checked"});

		# display varbinds and their format string
		for (my $i = 0; $i < scalar(@traps) ; $i++) {
			my %setting = (VARNAME=>$traps[$i], NUM=>$i + 1);
			print template("config_row.tpl", \%setting);
		}

		my @alarms = split($delimiter, $ini->val($alarmsection, $trapnum));
		print template("config_format.tpl", 
				{FORMAT=>$formats[0], CORR=>$alarms[0]});

		# display varbinds for the trap for selection as severity varbind
		my $target = $alarms[1];

		# check if this is a fixed severity or a varbind
		my $fixed = 0;
		foreach my $sev (@severities) {
			if ($target eq $sev) {
				$fixed = 1;
				last;
			}
		}

		# list varbinds for selection as severity variable
		foreach my $name (@traps) {
			if (defined($target) && ($target eq $name)) {
				printf "<option selected>%s</option>\n", $name;
			}
			else {
				printf "<option>%s</option>\n", $name;
			}
		}

		print template("config_alarm_fixed_sev.tpl");
		foreach my $name (@severities) {
			if ($target eq $name) {
				printf "<option selected>%s</option>\n", $name;
			}
			else {
				printf "<option>%s</option>\n", $name;
			}
		}

		my %params = ();
		if (!$fixed) {

			# get the severity mapping
			splice @alarms, 0, 2;
			for (my $i = 0; $i < scalar(@severities) ; $i++) {
				$params{$severities[$i]} = $alarms[$i];
			}
		}
		print template("config_sev.tpl", \%params);

		# display alarm trap numbers for clearing event linkage
		my @alarms = $ini->Parameters($alarmsection);
		my @clearings = split($delimiter, $ini->val($clearsection, $trapnum));
		foreach my $alarm (@alarms) {
			if ($alarm eq $clearings[0]) {
				printf "<option selected>%s</option>\n", $alarm;
			}
			else {
				printf "<option>%s</option>\n", $alarm;
			}
		}

		print template("config_clear_fixed_sev.tpl");
		foreach my $name (@severities) {
			if ($clearings[1] eq $name) {
				printf "<option selected>%s</option>\n", $name;
			}
			else {
				printf "<option>%s</option>\n", $name;
			}
		}
		print
			template("config_trailer.tpl", 
				{MIBNAME=>$Q::mibname, TRAPNUM=>$trapnum }),
			$q->end_form();
	}
	else {
		print
			$q->start_multipart_form, template("edit_header.tpl");

		my @traps = $ini->Parameters($trapsection);
		foreach my $trap (@traps) {
			my @vals = split($delimiter, $ini->val($trapsection, $trap));
			my @formats = split($delimiter, $ini->val($formatsection, $trap));
			print template("edit_row.tpl", 
				{TRAPNUM=>$trap, TRAPNAME=>$vals[0], EVENTTYPE=>$formats[0]});
		}

		print
			template("edit_trailer.tpl", {"MIBNAME"=>$Q::mibname}),
			$q->end_form();
	}
}

# ******************************************************************
# edit indivdual trap processing
sub configfile {

	my($q, $ini) = @_;
	$q->import_names("Q");
	if ($Q::Cancel) {
		editfile($q);
		return;
	}

	print 
		template("header.tpl", {DESC => "Edit File"}),
		template("menu2.tpl", {MIBNAME=>$Q::mibname}); 

	# get config info
	if (!$ini) {
		$ini = new Config::IniFiles(-file=>$datadir . $Q::mibname . ".conf");
	}

	if ($Q::ID ne "CONFIGFILE") {
		#process edited info
		print $notimplemented;
		# get trap number
	}
	else {
		# make sure we call linktraps before updating anything
		#if ($q->param("radio") ne $ignore_state) {
			linktraps($q, $ini);
		#}

		# save the info back into the config file
		$ini->setval($formatsection, $Q::trapnum, 
			$q->param("radio") . $delimiter . $Q::format);


		$ini->RewriteConfig;
		editfile($q);

	}
}

# ******************************************************************
# link up alarm and clearing traps
sub linktraps {

	my($q, $ini) = @_;

	$q->import_names("Q");
	track_state_change($ini, $Q::trapnum, $Q::radio);

	if ($Q::radio eq $clear_state) {

		my $val = $Q::list . $delimiter . $Q::clearoverride;
		$ini->AddSection($clearsection);
		if (!$ini->setval($clearsection, $Q::trapnum, $val)) {
			$ini->newval($clearsection, $Q::trapnum, $val)
		}
	}
	elsif ($Q::radio eq $alarm_state) {

		# save alarm-specifc info
		my @vals = ();
		foreach my $sev (@severities) {
			push @vals, $q->param($sev);
		}

		my $val = $Q::corr . $delimiter;
		if (length($Q::alarmoverride) > 0) {
			$val .=  $Q::alarmoverride;
		}
		else {
			$val .= $delimter . $Q::varbind . $delimiter . join($delimiter, @vals);
		}
		$ini->AddSection($alarmsection);
		if (!$ini->setval($alarmsection, $Q::trapnum, $val)) {
			$ini->newval($alarmsection, $Q::trapnum, $val)
		}
	}
	#$ini->RewriteConfig;

}

# ******************************************************************
# select config processing
sub selectfile {

	my($q) = @_;
	$q->import_names("Q");


	#printf "id=%s mibname=%s\n", $Q::ID, $Q::mibname;
	if ($Q::ID eq "SELECTFILE") {
		#process edited info
		$q->param($mibname, $q->param("select"));
		editfile($q);
	}
	else {

		# get config file names
		local *DIR;
		opendir(DIR, $datadir);
		my @files = grep { /\.conf$/ } readdir(DIR);
		closedir(DIR);

		if (scalar(@files) <= 0) {
			uploadfile($q);
			return;
		}

		print 
			template("header.tpl", {DESC => "Select File"}),
			template("menu.tpl"); 

		print
			$q->start_multipart_form, template("select_header.tpl");

		for (my $i = 0; $i < scalar(@files) ; $i++) {
			$files[$i] =~ s/\.conf$//;
			printf "<option>%s</option>\n", $files[$i];
		}

		print
			template("select_trailer.tpl", ("MIBNAME"=>$Q::mibname)),
			$q->end_form();
	}
}


# ******************************************************************
# generate script and trapd.conf
sub generate {

	my($q) = @_;
	$q->import_names("Q");

	# get config info
	if (!$ini) {
		$ini = new Config::IniFiles(-file=>$datadir . $Q::mibname . ".conf");
	}

	print 
		template("header.tpl", {DESC =>"Generate Script" }),
		template("menu2.tpl", {"MIBNAME"=>$Q::mibname});

	if ($Q::ID ne "GENERATE") {

		my %hash = ();
		# save opcinfo
		foreach my $key (@opcinfo) {
			my $trans = $q->param($key);
			$ini->setval($opcinfosection, $key, $trans);
			if ($key eq "nodename") {
				$trans =~ s/\$(\d+)/~~$1~~/g;
			}
			# put opcinfo in hash for incorporating into script
			$hash{$key} = $trans;
		}
		$ini->RewriteConfig;

		my $app = new Config::IniFiles(-file=>$appini);
		$hash{$opcmsg} = $app->val("app", $opcmsg);

		my @errmsg;
		my $ok;
		#my $text = "Generate Script";
		($ok, @errmsg) = validate_config($ini);

		if ($ok) {
			$hash{TRAPDESCR} = generate_script($ini);
			my $partialname = $app->val("app", "scriptdir") . $Q::mibname;
			my $scriptname = $partialname  . ".pl";
			$hash{SCRIPTNAME} = $scriptname;


			# save script
			local *F;
			open(F, "> $scriptname") || return;

			# customize script
			print F template("script.tpl", \%hash);
			close(F);
			push @errmsg, "Script file generated: $scriptname";

			# generate partial trapd.conf file for replacing corresponding sections
			my $confname = $partialname . ".trapd.conf";
			generate_trapd_conf($ini, $Q::mibname, $app->val("app", "trapdconf"), $confname );
			push @errmsg, "Partial trapd.conf file generated: $confname";

			# add/update perl script name in trusted command list
			my $basename = $Q::mibname . ".pl";
			if (update_trusted_command($app->val("app", "trustedcmddir"), 
				$basename, 
				$app->val("app", "proddir") . $basename
				)) {
				push @errmsg, "Trusted command file updated: $myappname";
			}
		}

		print "<h4>";
		foreach my $msg (@errmsg) {
			printf "%s<br>\n", $msg;
		}
		print "</h4>";

		# debug only
		# print $hash{TRAPDESCR};

	}
	else {
		if (!$ini->SectionExists($opcinfosection)) {
			# create empty section
			$ini->AddSection($opcinfosection);
			foreach my $key (@opcinfo) {
				$ini->newval($opcinfosection, $key, "");
			}
			$ini->RewriteConfig;
		}

		my %hash = (MIBNAME=>$Q::mibname);
		foreach my $key (@opcinfo) {
			my $text = $ini->val($opcinfosection, $key);
			if (length($text) > 0) {
				$hash{$key} = $text;
			}
		}

		print
			$q->start_multipart_form, 
			template("generate_header.tpl", \%hash),
			$q->end_form();
	}
}


# ******************************************************************
# start html header
sub start_doco {
	my($q) = @_;
	print $q->start_html("Trap Customization Application");

}


# ******************************************************************
# end html document
sub end_doco {
	my($q) = @_;
	print template("footer.tpl");

	print $q->end_html;
}

# *******************************************************************
# expand template
sub template {
	my ($filename, $fillings) = @_;
	my $text;
	local $/;
	local *IN;
	open(IN, "< $filename\0") || return;
	$text = <IN>;
	close(IN);
	# replace quoted words with value in %$fillings hash
	$text =~ s{%%(.*?)%%}
			{exists($fillings->{$1})
				? $fillings->{$1}
				: ""
		}gsex;
	return $text;
}

# ******************************************************************
# copy uploaded text file
# 1st param is a handle, 2nd the output file name
sub copy_text_file {
	my ($in, $filename) = @_;

	local *OUTPUT;
	open(OUTPUT, ">$filename");
	binmode $in;
	binmode OUTPUT;

	while (<$in>) {
		print OUTPUT;
	}

	close OUTPUT;
	close $in;
}

# ******************************************************************
# Extract trap definitions from MIB and save in config file
sub save_traps {
	my ($q, $fsm, $filename) = @_;

	# save the trap section
	my $ini = new Config::IniFiles;
	$ini->AddSection($trapsection);
	for (my $i = 0; $i < $fsm->getTotalTraps ; $i++) {
		$ini->newval($trapsection, 
			$fsm->getTrapNum($i),
			join($delimiter, $fsm->getTrapName($i), $fsm->getVars($i))
			);
	}
	$ini->WriteConfig($datadir . $filename . ".conf");
	#print "<H1>File uploaded</H1>";
	$q->param(-name=>$mibname, -value=>$filename);
	editfile($q, $ini);
}

# ******************************************************************
# Create the format section based on info in the trap section
sub create_format_secton {
	my ($ini) = @_;

	$ini->AddSection($formatsection);

	my @traps = $ini->Parameters($trapsection);
	foreach my $trap (@traps) {
		$ini->newval($formatsection, $trap, $ignore_state . $delimiter);
	}
	$ini->RewriteConfig;
}

# ******************************************************************
# make sure the info in the config file is updated according to the
# user changing the alarm/clearing state of a trap definition
sub track_state_change {
	my ($ini, $trapnum, $state) = @_;

	my @formats = split($delimiter, $ini->val($formatsection, $trapnum));
	my $laststate = shift @formats;

	if ($state eq $alarm_state) {
		if ($laststate eq $clear_state) {
			# clean up clearing section
			$ini->delval($clearsection, $trapnum);
		}
	}
	else {
		if ($laststate eq $alarm_state) {
			# clean up alarm section
			$ini->delval($alarmsection, $trapnum);

			# clean up existing links from clearing events
			my @keys = $ini->Parameters($clearsection);
			foreach my $key (@keys) {
				my @params = split($delimiter, $ini->val($clearsection, $key));
				if ($params[0] == $trapnum) {
					#$params[0] = "";
					#$ini->setval($clearsection, $key, join($delimiter, @params));
					$ini->delval($clearsection, $key);
				}
			}
		}
	}

}

# ******************************************************************
# perform necessary validation on the config file and return error
# messages
sub validate_config {
	my ($ini) = @_;

	my $result = 1;
	my @msgs = ();

	foreach my $section (@sections) {
		if (!$ini->SectionExists($section)) {
			push @msgs, "Error: $section section does not exist.";
			$result = 0;
		}
		else {
			my @keys = $ini->Parameters($section);
			if (!defined(@keys) || (scalar(@keys) <= 0)) {
				push @msgs, "Error: $section section contains no entries";
				$result = 0;
			}
		}
	}

	if ($result) {
		# check if a clearing event has been linked to an alarm
		my @traps = $ini->Parameters($clearsection);
		foreach my $trap (@traps) {
			my @clears = split($delimiter, $ini->val($clearsection, $trap));
			if ($clears[0] !~ /^\d+$/) {
				push @msgs, "Error: clearing event (Trap $trap) not linked to any alarm.";
				$result = 0;
			}
		}

		@traps = $ini->Parameters($formatsection);
		foreach my $trap (@traps) {
			my @vals = split($delimiter, $ini->val($formatsection, $trap));
			my $state = shift @vals;
			my $format = shift @vals;

			# check if the format contains reference to non-existing varbinds
			my @refs = parseParams($format);
			my @trapinfo = split($delimiter, $ini->val($trapsection, $trap));
			foreach my $pos (@refs) {
				if ($pos >= scalar(@trapinfo)) {
					push @msgs, "Error: message format for trap $trap references non-existent varbind \$$pos.";
					$result = 0;
				}
			}

			if ($state eq $alarm_state) {
				# check all alarms have been defined
				if (!defined($ini->val($alarmsection, $trap))) {
					push @msgs, "Error: alarm trap $trap not defined in alarm section.";
					$result = 0;
				}
				else {
					# check if the correlator format contains reference to non-existing varbinds
					my @alarminfo = split($delimiter, $ini->val($alarmsection, $trap));
					my @refs = parseParams($alarminfo[0]);
					foreach my $pos (@refs) {
						if ($pos >= scalar(@trapinfo)) {
							push @msgs, "Error: correlator format for alarm trap $trap references non-existent varbind \$$pos.";
							$result = 0;
						}
					}
				}
			}
			elsif ($state eq $clear_state) {
				# check all clearing events have been defined
				my @defs = split($delimiter, $ini->val($clearsection, $trap));
				if (!defined(@defs) || (scalar(@defs) <= 0)) {
					push @msgs, "Error: clearing Trap $trap not defined in clearing section";
					$result = 0;
				}
				else {
					# check that clearing event has all the info defined
					if (scalar(@defs) < 2) {
						push @msgs, "Error: Clearing $trap configuration incomplete: alarm to clear or severity mapping missing";
						$result = 0;
					}

					# check that the alarm's defined correlator can be used in by this clearing event
					my @alarminfo = split($delimiter, $ini->val($alarmsection, $defs[0]));
					my @alarmtrapinfo = split($delimiter, $ini->val($trapsection, $defs[0]));
					my @cleartrapinfo = split($delimiter, $ini->val($trapsection, $trap));
					my @refs = parseParams($alarminfo[0]);
					foreach my $pos (@refs) {
						if ($pos >= scalar(@cleartrapinfo)) {
							push @msgs, "Error: correlator format for clearing trap $trap references non-existent varbind \$$pos.";
							$result = 0;
						}
						else {
							# check if the parameters refer to the same varbinds
							if (($pos != 0) && ($alarmtrapinfo[$pos] ne $cleartrapinfo[$pos])) {
								push @msgs, "Error: correlator format references different varbinds for alarm trap $defs[0] ".
									"and clearing trap $trap: $alarmtrapinfo[$pos] vs $cleartrapinfo[$pos]";
								$result = 0;
							}
						}
					}

				}
			}

		}
		# check alarm severity mapping has been defined
		@traps = $ini->Parameters($alarmsection);
		foreach my $trap (@traps) {
			my @vals = split($delimiter, $ini->val($alarmsection, $trap));
			if (scalar(@vals) < 2) {
				push @msgs, "Error: alarm $trap configuration incomplete: correlator or severity mapping missing";
				$result = 0;
			}
		}
	}

	my $app = new Config::IniFiles(-file=>$appini);
	if (!defined($app)) {
		push @msgs, "Error: application .ini file not found: $appini";
		$result = 0;
	}
	else {
		my $dir = $app->val("app", "scriptdir");
		if (!-d $dir) {
			push @msgs, "Error: script directory not found: $dir";
			$result = 0;
		}
	}
	
	if (!$result) {
		push @msgs, "Please fix problem(s) and try again!";
	}

	return ($result, @msgs);
}

# ******************************************************************
# generate an automation script based on the config info
# It is assumed that the config has been validated before
# this function is called.
sub	generate_script {
	my ($ini) = @_;

	my %hash = ();
	my @text;
	# generate trap description section
	my @traps = $ini->Parameters($formatsection);
	foreach my $trap (@traps) {
		my @mappings = ();
		my @vals = split($delimiter, $ini->val($formatsection, $trap));
		if ($vals[0] ne $ignore_state) {
			my $format= $vals[1];
			my $corr = "";
			my $sev = "";
			if ($vals[0] eq $alarm_state) {
				my @params = split($delimiter, $ini->val($alarmsection, $trap));
				$corr = $params[0];
				my $varbind = $sev = $params[1];
				# lookup the varbind position in the trap section
				my @vars = split($delimiter, $ini->val($trapsection, $trap));
				for (my $i = 1; $i < scalar(@vars) ; $i++) {
					if ($varbind eq $vars[$i]) {
						$sev = $i;

						# define mapping for varbind
						for (my $j = 0; $j < scalar(@severities) ; $j++) {
							my @defs = split(",", $params[$j + 2]);
							foreach my $state (@defs) {
								push @mappings, sprintf($hash_format, $state, $severities[$j]);
							}
						}

						last;
					}
				}
			} 
			elsif ($vals[0] eq $clear_state) {
				# get clearing defs
				my @clearings = split($delimiter, $ini->val($clearsection, $trap));
				$sev = $clearings[1];
				my @params = split($delimiter, $ini->val($alarmsection, $clearings[0]));
				$corr = $params[0];
			}

			# translate all $N to %%N%% for the automated script to work
			my $mapping = "undef";
			if (scalar(@mappings)) {
				$mapping = "{" . join(",", @mappings) . "}";
			}
			my $trans = sprintf($trapdesc_format, $trap, $format, $corr, $sev, $mapping);
			$trans =~ s/\$(\d+)/~~$1~~/g;

			push @text, $trans;
		}
	} 

	return join("\n", @text);

	return %hash;
}

# ******************************************************************
# generate partial trapd.conf file
sub generate_trapd_conf {
	my ($ini, $mibname, $infile, $outfile) = @_;
	local *OUT;
	open (OUT, "> $outfile") || return;

	# do it for every trap defined
	my @traps = $ini->Parameters($formatsection);
	my $cmd = sprintf q(%s.trapd.conf $S $A $>0), $mibname;
	foreach my $trap (@traps) {
		my @formats = split($delimiter, $ini->val($formatsection, $trap));
		my @names = split($delimiter, $ini->val($trapsection, $trap));
		my @text = update_trapdconf_by_trap($infile, $names[0], 
			($formats[0] ne $ignore_state)? $cmd: undef);
		foreach my $line (@text) {
			print OUT $line;
		}
	}
	close(OUT);

}

# ******************************************************************
# generate partial trapd.conf file
sub update_trapdconf_by_trap {
	my ($inFile, $trapname, $cmd) = @_;
	local *IN;
	my @text = ();
	open(IN, "< $inFile") || return;
	# customize EXEC
	my $insert = 1;
	while (<IN>) {
		if (/$trapname/ .. /EDESC/) {
			if (/$trapname/ .. /SDESC/) {
				my @vals = split(" ", $_);
				if ($vals[0] eq "EXEC") {
					# replace action
					if (defined($cmd)) {
						push @text, "EXEC $cmd\n";
					}
					$insert = 0;
				}
				elsif (($vals[0] eq "SDESC") && $insert) {
					# replace action
					if (defined($cmd)) {
						push @text, "EXEC $cmd\n";
					}
					push @text, $_;
				}
				else {
					push @text, $_;
				}
			}
			else {
				push @text, $_;
			}
		}
	}
	close(IN);

	return @text;

}

# ******************************************************************
# update the trusted command to include, if necessary, the perl script name
sub update_trusted_command {
	my ($path, $key, $val) = @_;

	local *IN, *OUT;
	my $name = $path . $myappname;
	my $noentry = 1;
	if (-f $name) {
		my $tmpfile = $name . "tmp";

		open(IN, "< $name");
		open(OUT, "> $tmpfile");

		while (<IN>) {
			if (/^$key=/) {
				#print OUT $key . "=", $val;
				$noentry = 0;
			}
			else {
				print OUT;
			}
		}
		if ($noentry) {
			print OUT $key . "=", $val;
		}

		close(IN);
		close(OUT);

		if ($noentry) {
			unlink($name);
			rename($tmpfile, $name);
		}
		else {
			unlink($tmpfile);
		}
	}
	else {
		# create new file
		open(OUT, "> $name");
		print OUT $key . "=" . $val;
		close(OUT);
	}

	return $noentry;
}

# ******************************************************************
# returns an array of all positional parameters in the text string passed
# as the first parameter to this function. If the text contains $1,$3, this
# function returns an array of [1, 3]
sub parseParams {
	my ($text) = @_;
	my @result = ();

	my $i = 1;
	while (1) {
		if ($text =~ /(\$(\d+)[^\$]*){$i}/) {
			push @result, $2;
		}
		else {
			last;
		}
		$i++;
	}

	return @result;

}
